## -*-Tcl-*-
 # ==========================================================================
 #  Vince's Additions - an extension package for Alpha
 # 
 #  FILE: "wwwRender.tcl"
 #                                    created: 04/30/1997 {11:04:46 am} 
 #                                last update: 05/20/2002 {09:58:50 AM} 
 #  Author: Vince Darley
 #  E-mail: <vince@santafe.edu>
 #    mail: 317 Paseo de Peralta, Santa Fe, NM 87501, USA
 #     www: <http://www.santafe.edu/~vince/>
 #     
 #  Description:
 #  
 #  Procedures to fetch remote and then render local html files.
 #  
 #  See the "wwwVersionHistory.tcl" file for license info, credits, etc.
 # ==========================================================================
 ##

# Make sure that the wwwMode.tcl file has been loaded.
wwwMode.tcl

proc wwwRender.tcl {} {}

namespace eval WWW {}

# ===========================================================================
# 
#  ----  #
# 
#  Rendering Files  #
# 

## 
 # -------------------------------------------------------------------------
 # 
 # "WWW::renderUrl" --
 #  
 # Attempt to render a url if the http package is available and the url is
 # actually remote.  If local, convert the filename and simply pass it on to
 # 'WWW::renderFile'
 #  
 # -------------------------------------------------------------------------
 ##

proc WWW::renderUrl {{url ""} {fileName ""}} {

    global WWWmodeVars WWW::FileNumber WWW::UrlSource file::separator mode

    set url [string trim [string trim $url {\"}]]
    if {![string length $url]} {
	if {![WWW::httpAllowed]} {
	    return [WWW::renderFile]
	} elseif {$mode == "WWW" && [info exists WWW::UrlSource([win::Current])]} {
	    set url [set WWW::UrlSource([win::Current])]
	} elseif {[catch {getSelect} url]} {
	    set url $WWWmodeVars(homePage)
	}
	if {[catch {dialog::getUrl "Enter the url to be viewed:" $url} url]} {
	    status::errorMsg "Cancelled."
	}
    }
    if {![regexp {^[^:]+:/[/]+} $url]} {set url http://$url} 
    if {[regexp {^file://} $url]} {
	WWW::massagePath  url
	return [WWW::renderFile $url]
    } elseif {[regexp {^ftp://} $url]} {
	WWW::ftpLink $url
    } elseif {![WWW::httpAllowed]} {
	# We can't handle this yet.
	WWW::externalLink $url
	return
    } 
    # Make sure that this url has a trailing '/' if it is a server.
    set parent [string range $url 0 [string last "/" $url]]
    if {[regexp {^[^:]+:/[/]+$} $parent]} {append url "/"}
    # Check to make sure that we don't already have this window, or at least
    # the file (so that we don't have to download it again.)
    foreach name [array names WWW::UrlSource] {
	if {[set WWW::UrlSource($name)] == $url} {
	    # Hmmm...  is it a window or a file?
	    if {![catch {bringToFront $name}]} {
		return
	    } elseif {[file isfile $name]} {
		return [WWW::renderFile $name]
	    } 
	} 
    }
    if {![string length $fileName]} {
	ensureset WWW::FileNumber 0
	incr WWW::FileNumber
	set fileName "WWW-[set WWW::FileNumber].html"
    } 
    set newFile [file join [temp::directory WWW-tmp] $fileName]
    set WWW::UrlSource([file nativename $newFile]) $url
    status::msg "Fetching $url"
    url::fetch $url [temp::directory WWW-tmp] $fileName
    WWW::renderFile $newFile
}

## 
 # -------------------------------------------------------------------------
 # 
 # "WWW::renderFile" --
 #  
 # Attempt to render a local file if it is actually html code, otherwise just
 # insert the entire contents of the file in a new window.  We go to a bit of
 # trouble here to ensure that the title is unique, and set a bunch of
 # information for the title that will be used elsewhere.
 #  
 # -------------------------------------------------------------------------
 ##

proc WWW::renderFile {{f ""} {title ""}} {
    
    # How long will this take?
    set time1 [ticks]
    
    # Preliminaries
    if {[string length $title] && ![catch {bringToFront $title}]} {
	# We already have this window open.
	return $title
    }
    if {$f == "" || [file isdir $f]} {
	if {[catch {getfile "View which file?" $f} f]} {
	    status::errorMsg "Cancelled."
	} 
    }
    if {![file isfile $f]} {
        dialog::errorAlert "Cancelled -- couldn't find the file '$f' !!"
    } 

    global WWW::BaseUrl WWW::UrlSource WWW::FileSource mode WWWmodeVars
    global WWW::ParsingString WWW::LinkTargetWindow

    WWW::massagePath f
    set t [file::readAll $f]
    # Is this an html file?
    set pat    "(<((!DOCTYPE\[\r\n\t \]+)?HTML|BODY)\[^>\]*>)"
    set isHtml [regexp -nocase $pat $t]
    # Try to determine what the title should be.
    if {![string length [string trim $title]]} {
	if {$isHtml} {
	    regexp -nocase {<TITLE[^>]*>(.*)</TITLE>} $t dummy title
	} elseif {[info exists WWW::UrlSource($f)]} {
	    set url [string trimright [set WWW::UrlSource($f)] "/"]
	    regsub {\?.+$} $url {} url
	    set last  [string last "/" $url]
	    set title [string range $url [expr $last + 1] end]
	} else {
	    set title [file tail $f]
	}
    }
    if {![string length [string trim $title]]} {set title "(no title)"}
    # Massage the title so it works in the menus.
    regsub -all "\[\r\n\t \]+" $title { } title
    set title [win::MakeTitle [quote::Unhtml [string trim $title]]]
    regsub -all {/}        $title {-} title
    regsub -all {[\{\(\<]} $title {} title
    regsub -all {[\}\)\>]} $title {} title
    set title [string trim $title]
    # Determine what the base is for this file.
    set pat "<BASE\[\r\n\t \]+HREF\[\r\n\t \]*=\[\r\n\t \]*\"*(\[^\"\]*)\"*\[^>\]*>"
    if {![regexp -nocase $pat $t dummy base]} {
	if {[info exists WWW::UrlSource($f)]} {
	    # This should always supercede.
	    set base [set WWW::UrlSource($f)]
	} else {
	    set base [file::toUrl $f]
	}
    }
    regsub -all {\\} $base {/} base
    # Make sure that we're not dealing with a title that has already been
    # registered, but from a different source.
    if {[info exists WWW::BaseUrl($title)]} {
	if {$base != [set WWW::BaseUrl($title)]} {
	    # 2 titles, different bases.  We'll create a different title
	    # for this page.
	    while {1} {
		if {![info exists "WWW::BaseUrl($title )"]} {
		    set title "$title "
		    break
		} elseif {[set "WWW::BaseUrl($title )"] == $base} {
		    set title "$title "
		    break
		} else {
		    set title "$title "
		}
	    }
	}
    }
    set WWW::BaseUrl($title) $base
    if {![info exists WWW::FileSource($title)]} {
	set WWW::FileSource($title) $f
    }
    if {[info exists WWW::UrlSource($f)]} {
        set WWW::UrlSource($title) [set WWW::UrlSource($f)]
    } 
    regsub -all {\\} [set WWW::BaseUrl($title)] {/} WWW::BaseUrl($title)
    # Determine what the base targets are for these links.
    set pat "<BASE\[\r\n\t \]+TARGET\[\r\n\t \]*=\[\r\n\t \]*\"*(\[^\"\]*)\"*\[^>\]*>"
    if {[regexp -nocase $pat $t dummy target]} {
	set WWW::LinkTargetWindow($title) $target
    } else {
	set WWW::LinkTargetWindow($title) ""
    }
    # If this web page uses frames ...
    set pat "<FRAMESET(\[^>\]*)>(.+)</FRAMESET>"
    if {[regexp -nocase $pat $t]} {
	return [WWW::renderFrames $title $t]
    }
    # If this title is not already a window ...
    if {[catch {bringToFront $title}]} {
	# ... then create a new shell window, and add the parsed text.
	status::msg "Rendering"
	set title [WWW::createNewWindow $title]
	if {!$isHtml} {
	    # Not an html file.
	    WWW::cacheText $t
	} else {
	    insertText -w $title "\r\rPlease wait ... rendering html window ...\r"
	    regexp -nocase {<HTML[^>]*>(.*)</HTML>} $t dummy t
	    regexp -nocase {<BODY[^>]*>(.*)</BODY>} $t dummy t
	    catch {WWW::removeCrap t}
	    WWW::parseHtml $t $title [WWW::setFillColumn $title]
	}
	replaceText -w $title [minPos] [maxPos] [set WWW::ParsingString]\r\r
	catch {WWW::hyperizeColorsLinks $title}
	setWinInfo -w $title read-only 1	
	goto [minPos]
	refresh
	status::msg ""
    }
    WWW::visitedLink [set WWW::BaseUrl($title)]
    WWW::addHistoryItem $f
    if {$WWWmodeVars(autoFloatMarks)} {catch floatNamedMarks}
    # How long did this take?
    set time2 [ticks]
    set time3 [expr {($time2 - $time1) / 6}] 
#     alertnote "[expr {$time3 / 10.0}] seconds"
    return $title
}

## 
 # -------------------------------------------------------------------------
 # 
 # "WWW::renderFrames" --
 #  
 # We attempt to set different window sizes for each frame.  By setting
 # values in the 'WWW::LinkWindowTarget' array, we're designating links from
 # one frame to be rendered in a different window, using the magic in
 # "wwwMenu.tcl" for determining when WWW browser windows should be killed.
 # 
 # Known limitations:
 # 
 #  Nested <FRAMESET> are not dealt with at all.
 # 
 #  Should come up with some method for offering <NOFRAMESET> option if the
 #   tag exists.
 # 
 #  If the COLS or ROWS option is a numeric value (i.e. no %), we attempt to
 #   size the windows according to the code.  Could also try to do something
 #   fancy with % values, but that will require a bit more work here -- what
 #   we might need to do is rethink that whole 'window parameters' settings
 #   so that we just have one (hopefully wide) window setting that isn't
 #   likely to change without more explicit action from the user, such as a
 #   'Default Window Size" menu item in the WWW menu, which would open a
 #   window with instructions and a hyperlink to save settings.  It all gets
 #   more complicated with synching the behavior between Alpha7/8/X and
 #   Alphatk.
 #   
 #  Links created with <A HREF...  TARGET...> store the target info in the
 #   array WWW::Links, but 'WWW::link' doesn't do anything with this info. 
 #   Currently, the target is entirely window dependent, i.e. target windows
 #   are designated for all or no links in the window.
 # 
 # -------------------------------------------------------------------------
 ##

proc WWW::renderFrames {title t} {
    
    global WWW::BaseUrl WWWmodeVars WWW::LinkWindowTarget
    
    set base [set WWW::BaseUrl($title)]
    WWW::removeCrap t
    # Make sure that we were properly called.
    set pat "<FRAMESET\[^>\]*>(.+)</FRAMESET>"
    if {![regexp -nocase $pat $t dummy frames]} {
	error "'$title' shouldn't have been passed to WWW::renderFrames" 
    }
    # Determine any FRAMESET options
    set frameType ""
    set values [list]
    set pat "<FRAMESET(\[^>\]*)>"
    if {[regexp -nocase $pat $t dummy framesetArgs]} {
	set pat1 "COLS\[\r\n\t \]*=\[\r\n\t \"\]*(\[^\r\n\t \">\]+)"
	set pat2 "ROWS\[\r\n\t \]*=\[\r\n\t \"\]*(\[^\r\n\t \">\]+)"
	if {[regexp -nocase $pat1 $framesetArgs dummy cols]} {
	    set frameType "col"
	    set values [split $cols ","]
	} elseif {[regexp -nocase $pat2 $framesetArgs dummy rows]} {
	    set frameType "row"
	    set values [split $rows ","]
	} 
    }
    # Temporarily turn off 'autoFloatMarks'
    set oldAFM $WWWmodeVars(autoFloatMarks)
    set WWWmodeVars(autoFloatMarks) 0
    # Determine what the urls for each frame is.
    set pat1 "<FRAME\[^>\]+SRC\[\r\n\t \]*=\[\r\n\t \"\]*(\[^\r\n\t \">\]+)\[^>\]*>"
    set pat2 "<FRAME\[^>\]+NAME\[\r\n\t \]*=\[\r\n\t \"\]*(\[^\r\n\t \">\]+)"
    while {[regexp -nocase $pat1 $frames allofit frame]} {
        set frame [string trim [string trim $frame] "\""]
	if {[catch {url::makeAbsolute $base $frame} url]} {
	    set url ????://$frame
	} 
	lappend results $url
	# Determine if this is a target window.
	if {[regexp -nocase $pat2 $allofit dummy name]} {
	    set targetWindow($url) [string trim [string trim $name] "\""]
	}
	regsub $allofit $frames {} frames
    }
    # Let the user know what's going on, and only render the urls specified. 
    # If we're not going to do all of all the frames, then we won't resize
    # any of them later.
    if {![info exists results]} {
	set WWWmodeVars(autoFloatMarks)     $oldAFM
        dialog::errorAlert "This web page uses frames,\
	  but no frame pages could be identified."
    } elseif {![dialog::yesno -y "Ok" -n "Choose Frames" -c \
      "This web page uses frames, each of which will be opened\
      in a separate window."]} {
	foreach page $results {
	    set pageView [dialog::specialView::url $page]
	    set pages($pageView) $page
	}
	set p "View which frames?"
	set l [lsort -ignore [array names pages]]
	if {[catch {listpick -p $p -l $l} newResults]} {
	    status::errorMsg "Cancelled."
	} 
	set results [list]
	foreach pageView $newResults {
	    lappend results $pages($pageView)
	}
	set values [list]
    }
    # We'll kill the first window if necessary, but not the others.
    set title1 [WWW::renderUrl [lindex $results 0]]
    if {[info exists targetWindow([lindex $results 0])]} {
	set WWW::LinkWindowTarget($title1) $targetWindow([lindex $results 0])]
    } 
    switch $frameType {
	"col" {
	    # This is a side by side frame.
	    if {[is::Integer [set w [lindex $values 0]]]} {
		set g [getGeometry $title1] ; # l t w h
		sizeWin $title1 $w [lindex $g 3]
	    }
	}
	"row" {
	    # This is a row by row frame.
	    if {[is::Integer [set h [lindex $values 0]]]} {
		set g [getGeometry $title1] ; # l t w h
		sizeWin $title1 [lindex $g 2] $h 
	    } 
	}
	"" {}
	default {error "Unknown frame type: $frameType}
    }
    # Now render the remaining frames.
    set oldLONW $WWWmodeVars(linksOpenNewWindow)
    set WWWmodeVars(linksOpenNewWindow) 1
    foreach url [lrange $results 1 end] {
	set title [WWW::renderUrl $url]
	switch $frameType {
	    "col" {
		# This is a side by side frame.
		if {[is::Integer [set l [lindex $values 0]]]} {
		    set g [getGeometry $title] ; # l t w h
		    moveWin $title [expr {$l + 12}] [lindex $g 1]
		} 
	    }
	    "row" {
		# This is a row by row frame.
		if {[is::Integer [set t [lindex $values 0]]]} {
		    set g [getGeometry $title] ; # l t w h
		    moveWin $title [lindex $g 0] [expr {$t + 25}]
		} 
	    }
	}
	if {[info exists targetWindow($url)]} {
	    set WWW::LinkWindowTarget($targetWindow($url)) $title
	}
	WWW::setWindowParameters
    }
    set WWWmodeVars(linksOpenNewWindow) $oldLONW
    set WWWmodeVars(autoFloatMarks)     $oldAFM
    bringToFront $title1
    return $title1
}

## 
 # -------------------------------------------------------------------------
 # 
 # "WWW::removeCrap" --
 #  
 # Get rid of all of the stuff that we know that we can't deal with to make
 # the parsing go a little bit faster.
 #  
 # Feel free to add to this list, or find a way to deal with them !!
 # 
 # -------------------------------------------------------------------------
 ##

proc WWW::removeCrap {tt} {
    
    global WWWmodeVars

    upvar $tt t
    
    # This should help us out quite a bit.  Would this mean that we don't
    # ever have to include '\n' in 'regexp "\[\r\n\t \] ..." ???
    regsub -all {
} $t "\r" t

    # Get rid of all scripts and comments.
    WWW::substituteForTags {<SCRIPT}  {/SCRIPT>}  t
    WWW::substituteForTags {<!--}     {-->}       t
    WWW::substituteForTags {<STYLE}   {/STYLE>}   t
    WWW::substituteForTags {<COMMENT} {/COMMENT>} t
    # Get rid of the title, in case this page wasn't very well formed and
    # didn't have a proper body.
    WWW::substituteForTags {<TITLE}   {/TITLE>}   t

    # Get rid of other stuff that we know we can't handle, but ignore.
    regsub -nocase -all -- {<IMG[ \t\r\n]+SRC[ \t\r\n]*=[ \t\r\n]*\"[^\"]*bullet.gif\"[^>]*>} $t {} t
#   regsub -nocase -all -- "\[ \t\r\n\]+alt=\"(\[^\"\]*)\"\[^>\]*>" $t ">\\1<TD></A>" t
    regsub -nocase -all {<DIV[^>]*>}     $t {} t
    regsub -nocase -all {</DIV>}         $t {} t
    regsub -nocase -all {<FONT[^>]*>}    $t {} t
    regsub -nocase -all {</FONT>}        $t {} t
    regsub -nocase -all {<SPAN[^>]*>}    $t {} t
    regsub -nocase -all {</SPAN>}        $t {} t
    regsub -nocase -all {<TABLE[^>]*>}   $t {} t
    regsub -nocase -all {</TABLE>}       $t {} t
    regsub -nocase -all {<VAR>}          $t {} t
    regsub -nocase -all {</VAR>}         $t {} t
    # Since we're going to render these in plain text anyway, we'll get rid
    # of these here so that there are fewer iterations in 'WWW::parseHtml'.
    regsub -nocase -all {<CODE>}         $t {} t
    regsub -nocase -all {</CODE>}        $t {} t
    regsub -nocase -all {<KBD>}          $t {} t
    regsub -nocase -all {</KBD>}         $t {} t
    regsub -nocase -all {<SAMP>}         $t {} t
    regsub -nocase -all {</SAMP>}        $t {} t
    regsub -nocase -all {<TT>}           $t {} t
    regsub -nocase -all {</TT>}          $t {} t
    # Many web pages (ab)use empty table elements to visually format text.
    # Since we don't pay attention to them, we'll get rid of them.
    regsub -nocase -all {<TD[^>]*></TD>} $t {} t
    # Addresses will simply be in italics on a separate line.
    regsub -nocase -all {<ADDRESS>}      $t {<P><I>}   t
    regsub -nocase -all {</ADDRESS>}     $t {</I></P>} t
    
    # How does the user want to handle forms?
    if {$WWWmodeVars(ignoreForms)} {
	WWW::substituteForTags {<FORM} {/FORM>} t
    } else {
	regsub -nocase -all -- {<FORM[^>]*>}  $t {(form begins)} t
	regsub -nocase -all -- {</FORM[^>]*>} $t {(form ends)}   t
	WWW::substituteForTags {<SELECT} {/SELECT>} t {(option list)}
    }
    # How does the user want to handle images?
    if {$WWWmodeVars(ignoreImages)} {
	# We'll include links that use images instead of text, so long as
	# they've included an 'alt'.
	set pat1 "(<A HREF\[^>\]+>)\[\r\n\t \]*<img\[^>\]*alt\[\r\n\t =\]+\"(\[^\"\]+)\"\[^>\]*>\[\r\n\t \]*(</A>)"
	set pat2 "<img\[^>\]*>"
	regsub -nocase -all -- $pat1 $t {\1 \2 \3} t
	regsub -nocase -all -- $pat2 $t {} t
    } else {
	# We include links to all images, which might be for the actual image
	# file (in which case the link shows up as '[image]', or a 'real'
	# link to some other anchor.
	set pat1 "(<A HREF\[^>\]+>)\[\r\n\t \]*<img\[^>\]*alt\[\r\n\t =\]+\"(\[^\"\]+)\"\[^>\]*>\[\r\n\t \]*(</A>)"
	set pat2 "<img(\[^>\]*)src\[\r\n\t =\]+\"(\[^\"\]+)\"(\[^>\]*)>"
	set pat3 "<img\[^>\]*alt\[\r\n\t =\]+\"(\[^\"\]+)\"\[^>\]*>"
	set pat4 "<img\[^>\]*>" 
	regsub -nocase -all -- $pat1 $t {\1 \2 \3} t
	regsub -nocase -all -- $pat2 $t {<A HREF="\2"><img \1 \3 ></A> } t
	regsub -nocase -all -- $pat3 $t {[image: \1] } t
	regsub -nocase -all -- $pat4 $t {[image] } t
    }
    set t [string trim $t]
}

## 
 # -------------------------------------------------------------------------
 # 
 # "WWW::substituteForTags" --
 #  
 # Substitute for tags that aren't properly supported, possibly by doing some
 # sneaky substition for a tag that we can actually handle, or some info for
 # the user to know what might have been there.
 #  
 # -------------------------------------------------------------------------
 ##

proc WWW::substituteForTags {openTag closeTag _t {substitute ""}} {
    
    upvar $_t t

    set opentag  [string tolower $openTag]
    set OPENTAG  [string toupper $openTag]
    set closetag [string tolower $closeTag]
    set CLOSETAG [string toupper $closeTag]
    foreach tag [list $opentag $OPENTAG] {
	while {[set index0 [string first $tag $t]] != -1} {
	    set tagend [string first $closetag $t]
	    set TAGEND [string first $CLOSETAG $t]
	    if {$tagend < $index0} {set tagend "-1"} 
	    if {$TAGEND < $index0} {set TAGEND "-1"} 
	    if {$tagend == "-1" && $TAGEND == "-1"} {
		#alertnote "Unbalanced '$tag' tag"
		break
	    } elseif {$tagend != "-1" && $TAGEND != "-1"} {
		set index1 [expr {$tagend > $TAGEND} ? $TAGEND : $tagend]
	    } elseif {$tagend == "-1"} {
		set index1 $TAGEND
	    } else {
		set index1 $tagend
	    }
	    set index0 [expr $index0 - 1]
	    set index1 [expr $index1 + [string length $closeTag]]
	    set t0     [string range $t 0 $index0]
	    set t1     [string range $t $index1 end]
	    set t      ${t0}${substitute}${t1}
	}
    } 
}

proc WWW::hyperizeColorsLinks {title} {
    
    global WWW::ParsingCache
    
    foreach item [list Colors Marks Anchors] {
	ensureset WWW::ParsingCache($item) [list]
    }
    foreach item [list Links LinksReverse Anchors Marks] {
	global WWW::${item}
	set WWW::${item}($title) [list]
    }
    # Colors, hypertext, links
    foreach item [set WWW::ParsingCache(Colors)] {
	set pos0   [pos::math -w $title [minPos] + [lindex $item 0]]
	set pos1   [pos::math -w $title [minPos] + [lindex $item 1]]
	set color  [lindex $item 2]
	set cmd    [lindex $item 3]
	set link   [lindex $item 4]
	set target [lindex $item 5]
	text::color $pos0 $pos1 $color
	if {[string length $cmd]} {
	    # We attempt to avoid hyperlinking across empty space if
	    # possible.
	    set txt  [string trimleft [getText $pos0 $pos1]]
	    set pos0 [pos::math $pos1 - [string length $txt]]
	    text::hyper $pos0 $pos1 $cmd
	    lappend WWW::Links($title) [list $pos0 $pos1 $cmd $link $target]
	} 
    }
    set WWW::LinksReverse($title) [lreverse [set WWW::Links($title)]]
    # Marks
    foreach item [set WWW::ParsingCache(Marks)] {
	set name [lindex $item 0]
	set pos1 [pos::math -w $title [minPos] + [lindex $item 1]]
	set pos2 [pos::math -w $title [minPos] + [lindex $item 2]]
	set pos3 [pos::math -w $title [minPos] + [lindex $item 3]]
	setNamedMark $name $pos1 $pos2 $pos3
	lappend WWW::Marks($title) [list $name $pos1 $pos2 $pos3]
    }
    # Anchors
    foreach item [set WWW::ParsingCache(Anchors)] {
	set name [lindex $item 0]
	set pos1 [pos::math -w $title [minPos] + [lindex $item 1]]
	lappend WWW::Anchors($title) [list $name $pos1]
    }
    # Remove the cache now, unless it's needed for debugging.
    WWW::resetCache
}

# ===========================================================================
# 
# .