# -*-Tcl-*- (install) (nowrap)
# 
# File : ASprocessing.tcl
# 	        Created : 2002-03-10 11:54:39
#     Last modification : 2002-03-26 22:32:56
# Author : Bernard Desgraupes
# e-mail : <bdesgraupes@easyconnect.fr>
# Web-page : <http://webperso.easyconnect.fr/bdesgraupes/alpha.html>
# Description :
#     This file is part of the AppleScript mode package. It contains  the  script
#     processing procs.
#  
# (c) Copyright : Bernard Desgraupes, 2002
#     All rights reserved. This software is free software.  See  licensing  terms
#     in the AppleScript Help file.
#     


set scrp_params(resultwin) 	"* AppleScript Results *"

namespace eval Scrp {}

# Check that Tclapplescript shared library is present
# Find the "Extensions" folder
set extfold [tclAE::build::resultData 'MACS' core getd rtyp TEXT ---- [tclAE::build::propertyObject extn]]
# Load the Tclapplescript shared library
if {[expr [info tclversion] >= 8.0]} {
    if {[catch {load  [file join $extfold "Tool Command Language" Tclapplescript.shlb]}]} {
	alertnote "I could not find Tclapplescript.shlb in the folder \
	  '[file join $extfold "Tool Command Language"]'. AppleScript mode\
	  will fail to function correctly."
    } else {
	message "Tclapplescript library loaded."
    }
}

unset extfold


proc Scrp::openScript {} {
    if {[catch {Scrp::selectScript} scriptfile]} return 
    set text [Scrp::decompile $scriptfile]
}


proc Scrp::decompile {file {resID 128}} {
    global ScrpmodeVars
    set text [Scrp::doDecompile $file]
    new -n "[file tail $file].scpt" 
    insertText $text
    newMode Scrp
    if {$ScrpmodeVars(includeComments)} {
	Scrp::insertScriptComment [Scrp::getScriptComment $file]
    } 
}


proc Scrp::doDecompile {file {resID 128}} {
    set scrID [AppleScript load -rsrcid $resID $file]
    set text [encoding convertfrom macRoman [AppleScript decompile $scrID]]
    AppleScript delete context $scrID
    return $text
}


# This proc takes care of separating the descriptive comment
# from the rest of the script.
proc Scrp::compile {} {
    global HOME ScrpmodeVars
    set outFile [Scrp::makeScriptname [win::Current]]
    set commentlimits [Scrp::scriptCommentRange]
    set text [getText [lindex [lindex $commentlimits 0] 1] [maxPos]]
    Scrp::doCompile $outFile $text
    # Change creator to AppleScript Editor
    catch {AEBuild -r 'MACS' core setd ---- [tclAE::build::propertyObject fcrt \
      [tclAE::build::nameObject file [tclAE::build::TEXT $outFile]]] data ToyS}
    # Update the "Scripts Folder" submenu
    menu::buildScrpMenu
    if {$ScrpmodeVars(includeComments)} {
	Scrp::storeScriptComment [getText [lindex [lindex $commentlimits 1] 0] \
	  [lindex [lindex $commentlimits 1] 1]] $outFile
    } 
    message "Compiled $outFile"
}


proc Scrp::doCompile {outFile text {resID 128}} {
    set scrID [AppleScript compile -context 1 -- $text]
    AppleScript store -rsrcid $resID $scrID $outFile
    AppleScript delete context $scrID
}


proc Scrp::execute {} {
    if {[getPos]==[selEnd]} {
	set text [getText [minPos] [maxPos]]
    } else {
	set text [getText [getPos] [selEnd]]
    }
    set res [Scrp::doExecute $text]
    Scrp::displayResult "$res\r"
}


proc Scrp::doExecute {text} {
    return [AppleScript execute -- $text]
}


if {[expr [info tclversion] >= 8.0]} {
    proc Scrp::run {} {
	if {[catch {Scrp::selectScript} scrpath]} return 
	Scrp::displayResult "[Scrp::doRun $scrpath]\r"
    }
    
    
    proc Scrp::doRun {file {resID 128}} {
	set scrID [AppleScript load -rsrcid $resID $file]
	set res [AppleScript run $scrID]
	AppleScript delete context $scrID
	return $res
    }
    
} else {
    proc Scrp::run {} {
	catch {getfile "Select a script file"} scrpath
	if {$scrpath == "cancel"} return
	Scrp::displayResult "[Scrp::doRun $scrpath]\r"
    }
    
    
    proc Scrp::doRun {file} {
	set scptcode [AEBuild -r 'MACS' syso load ---- [tclAE::build::alis $file]]
	if {[regexp {scpt\([A-Fa-f0-9]+\)} $scptcode scptcode]} {
	    return [Scrp::trimResult [AEBuild -r 'MACS' syso dsct ---- $scptcode]]
	} else {
	    alertnote "Could not find an 'scpt' resource"
	}
    }
    
    proc Scrp::trimResult {res} {
 	    regsub "aevt\\\\ansr\{" $res "" res
	    regsub "'----':" $res "" res
	    regsub "errs:" $res "" res
	    regsub "errn:" $res "Error:" res
	    return [string trim $res "\{\}"]
    }
}


proc Scrp::runScript {name} {
    global HOME
    set scr [file join $HOME AppleScripts $name]
    Scrp::displayResult "[Scrp::doRun $scr]\r"
}


proc Scrp::makeScriptname {name} {
    global HOME
    set outFile $name
    # If the file does not exist on disk, create the 
    # compiled script in the AppleScripts folder
    if {![file exists $name]} {
	set outFile [file join $HOME AppleScripts [file tail $name]]
    }
    # If the name has an extension, drop it. Otherwise add an scpt extension
    if {[file extension $outFile]!=""} {
        set outFile [file root $outFile]
    } else {
	set outFile "$outFile.scpt"
    }
    # Shorten the name if longer than 32 chars
   set tail [file tail $outFile]
    if {[string length $tail]>=32} {
	set outFile [file join [file dir $outFile] \
	  "[string range $tail 0 24][string range $tail end-4 end]"]
    } 
    return $outFile
}


proc Scrp::displayResult {res} {
    global scrp_params
    if {$res=="\r"} {
        return
    } 
    catch {lsearch -exact [winNames] $scrp_params(resultwin)} indx
    if {[expr {$indx > -1}]} {
	bringToFront $scrp_params(resultwin)
    } else {
	new -n $scrp_params(resultwin)
    }
    goto [maxPos]
    insertText $res
}


proc Scrp::clearContexts {} {
    set contextsList [AppleScript info contexts]
    foreach cntxt $contextsList {
        if {$cntxt!="global"} {
            AppleScript delete context $cntxt
        } 
    } 
    message "All contexts deleted except 'global'"
}


# Script comments handling
proc Scrp::getScriptComment {file {resID 128}} {
    if {[catch {resource open $file} fileResId]} {
	alertnote "Error : $fileResId"
	return
    } 
    if {[catch {resource read TEXT [expr $resID + 1000] $fileResId} text]} {
	set text ""
    } 
    resource close $fileResId
    return $text
}


proc Scrp::insertScriptComment {comment} {
    if {$comment==""} {return} 
    goto [minPos]
    insertText "(* Description\r\t$comment\r*)\r"
    goto [minPos]
    ring::+
}


proc Scrp::storeScriptComment {txt outFile {resID 128}} {
    if {$txt==""} {
        return
    } 
    if {[catch {resource open $outFile w+} fileResId]} {
	alertnote "Error : $fileResId"
	return
    } 
    resource write -id [expr $resID + 1000] -file $fileResId -force TEXT $txt
    resource close $fileResId
}


proc Scrp::scriptCommentRange {} {
    set commentstart [minPos]
    set commenttextstart [minPos]
    set commentend [minPos]
    set commenttextend [minPos]
    set pos [minPos]
    if {![catch {search -s -f 1 -r 1 {\(\* Description[ \t\r]*} $pos} res]} {
	set commentstart [lindex $res 0]
	set commenttextstart [lindex $res 1]
	if {![catch {search -s -f 1 -r 1 {[ \t\r]*\*\)} $commenttextstart} res]} {
	    set commenttextend [lindex $res 0]
	    set commentend [lindex $res 1]
	}
    }
    return [list [list $commentstart $commentend] [list $commenttextstart $commenttextend]]
}


proc Scrp::addComment {} {
    Scrp::insertScriptComment ""
}


proc Scrp::lineContinuation {} {
    insertText "\r"
}


# If the Standard Additions are available, we use NavServices to select 
# exclusively files of type 'osas' or 'APPL'. Otherwise, use the 
# traditional getFile command.
proc Scrp::selectScript {} {
    # Dirty trick to check if Standard Additions are there
    set res [AEBuild -r 'MACS' syso rand]
    if {[regexp errn: $res]} {
	catch {getfile "Select a script file"} path
	if {$path=="cancel"} {error "User cancelled"}
	return $path
    } else {
	switchTo Finder
	set res [AEBuild -r 'MACS' syso stdf rtyp TEXT \
	  prmp [tclAE::build::TEXT "Select a script file"] \
	  ftyp [tclAE::build::List [list osas APPL] -untyped]]
	switchTo 'ALFA'
	if {[regexp errn: $res]} {error "User cancelled"}
	regexp {alis\([A-Fa-f0-9]+\)} $res data
	# Result is an alis. Coerce it to text.
	return [tclAE::build::resultData 'MACS' core getd ---- \
	  "obj {form:alis, want:cobj, from:'null'(), seld:$data}" rtyp TEXT]
    }
}



