# -*-Tcl-*- (install) (nowrap)
# 
# File : ASterminology.tcl
# 	        Created : 2002-03-10 11:54:39
#     Last modification : 2002-03-26 22:36:32
# Author : Bernard Desgraupes
# e-mail : <bdesgraupes@easyconnect.fr>
# Web-page : <http://webperso.easyconnect.fr/bdesgraupes/alpha.html>
# Description :
#     This file is part of the AppleScript mode package. It  contains  the  procs
#     to extract the scripting dictionary of an application.
#  
# (c) Copyright : Bernard Desgraupes, 2002
#     All rights reserved. This software is free software.  See  licensing  terms
#     in the AppleScript Help file.

namespace eval Scrp {}


# Initialization of variables
set scrp_params(symb) 		""
set scrp_params(outfileId) 	0
set scrp_params(infilepath)  	""

proc Scrp::initScrpParams {} {
    global scrp_params
    set scrp_params(aetepointer) 	0
    set scrp_params(nbAddParms) 	0
    set scrp_params(nbClasses) 		0
    set scrp_params(nbCompOps) 		0
    set scrp_params(nbElems) 		0
    set scrp_params(nbEnumerators)	0
    set scrp_params(nbEnums) 		0
    set scrp_params(nbEvents) 		0
    set scrp_params(nbKeyforms) 	0
    set scrp_params(nbPropr) 		0
    set scrp_params(nbSuites) 		0
}

Scrp::initScrpParams


proc Scrp::openDictionary {} {
    global scrp_params ScrpmodeVars aetevar
    catch {getfile "Get terminology from"} scrp_params(infilepath)
    if {$scrp_params(infilepath)=="cancel"} return
    if {[catch {resource open $scrp_params(infilepath)} fileResId]} {
	alertnote "Error : $fileResId"
	return
    } 
    set count 0
    if {$ScrpmodeVars(launchToGetTerminology)} {
	# Look for an scsz resource : its first byte contains the kLaunchToGetTerminology
	# flag (1 << 15). If set, launch the app and send a 'Get AETE' event (ascr/gdte).
	catch {resource list scsz $fileResId} scszList
	if {[string length [lindex $scszList 0]]!=0} {
	    catch {resource read scsz [lindex $scszList 0] $fileResId} scszbinvar
	    binary scan $scszbinvar H2 firstbyte
	    if {[eval expr {0x$firstbyte >> 7}]} {
		getFileInfo $scrp_params(infilepath) arr
		app::launchBack $arr(creator)
		set res [split [AEBuild -r '$arr(creator)' ascr gdte ---- 0] ,]
		set i 1
		set ext ""
		foreach item $res {
		    regexp {aete\(([A-Fa-f0-9]+)\)} $item dumm aetevar
		    if {[llength $res]>1} {
			set ext "_$i"
		    } 
		    if {[info exists aetevar] && $aetevar!=""} {
			Scrp::makeDictname $scrp_params(infilepath) $ext
			incr i
			Scrp::DictionaryConversion
			unset aetevar
			message "Dict written to $scrp_params(outfilepath)\r"
			edit -w $scrp_params(outfilepath)
			incr count
		    } 
		}
	    } 
	} 
    } 
    # If the previous failed, look for 'aete' resources
    if {!$count} {
	catch {resource list aete $fileResId} listrez
	# Store the hexadecimal data in a variable.
	if {[llength $listrez]!=0} {
	    set ext ""
	    foreach rezNum $listrez {
		if {[llength $listrez]>1} {
		    set ext "_$rezNum"
		}
		Scrp::makeDictname $scrp_params(infilepath) $ext
		catch {resource read aete $rezNum $fileResId} aetebinvar
		binary scan $aetebinvar H* aetevar
		Scrp::DictionaryConversion
		unset aetebinvar aetevar
		message "Dict written to '$scrp_params(outfilepath)'"
		edit -w $scrp_params(outfilepath)
	    }
	} else {
	    alertnote "Couldn't find terminology info for [file tail $scrp_params(infilepath)]"
	}
    } 
    resource close $fileResId
}

    
proc Scrp::makeDictname {file {ext ""}} {
    global scrp_params
    set scrp_params(outfilepath) "$scrp_params(infilepath).dict$ext"
    set tail [file tail $scrp_params(outfilepath)]
    if {[string length $tail]>32} {
	set scrp_params(outfilepath) [file join [file dir $scrp_params(outfilepath)]\
	  "[string range $tail 0 15][string range $tail end-13 end]"]
    } 
}


# Scripting terminology translation
# ---------------------------------

proc Scrp::DictionaryConversion {} {
    global scrp_params 
    if {[catch {open "$scrp_params(outfilepath)" w+} scrp_params(outfileId)]} {
	alertnote "Error : $scrp_params(outfileId)"
	return
    } 
    message "Creating [file tail $scrp_params(outfilepath)]"
    Scrp::doDictConversion
    flush $scrp_params(outfileId)
    close $scrp_params(outfileId)
}


proc Scrp::doDictConversion {} {
    global scrp_params 
    Scrp::initScrpParams
    Scrp::header
    # Suites loop
    for {set i 1} {$i <= $scrp_params(nbSuites)} {incr i} {
	Scrp::suite $i
    }
}


# Parsing procs
# -------------
# The eight first bytes are the header of the resource
proc Scrp::header {} {
    global scrp_params 
    puts -nonewline $scrp_params(outfileId) "Scripting terminology for \"[file tail $scrp_params(infilepath)]\"\r\r"
    puts -nonewline $scrp_params(outfileId) "Version : [Scrp::getNextVal].[Scrp::getNextVal]\r"  
    puts -nonewline $scrp_params(outfileId) "Language code: [Scrp::getNextVal 2]\nScript code: [Scrp::getNextVal 2]\r"
    set scrp_params(nbSuites) [Scrp::getNextVal 2]
    puts -nonewline $scrp_params(outfileId) "Number of Suites : $scrp_params(nbSuites)\r" 
}


# Parsing each suite data. From here we call the data for the events, classes,
# comparison operators and enumerations contained in a suite.
proc Scrp::suite {suitenum} {
    global scrp_params
    Scrp::outputLine "\r\r\r"
    Scrp::outputLine " -- "
    Scrp::checkAlignByte
    Scrp::outputLine " (" 4
    Scrp::outputVal ", level " 2
    Scrp::outputVal ", version " 2
	puts -nonewline $scrp_params(outfileId) ")" 
    set scrp_params(nbEvents) [Scrp::getNextVal 2]
    if {$scrp_params(nbEvents)} {
	puts -nonewline $scrp_params(outfileId) "\r\r\tTerms\r\t-----\r" 
    } 
    # Events Loop
    for {set i 1} {$i <= $scrp_params(nbEvents)} {incr i} {
	Scrp::event $i
    } 
    # Classes loop
    set scrp_params(nbClasses) [Scrp::getNextVal 2]
    if {$scrp_params(nbClasses)} {
	puts -nonewline $scrp_params(outfileId) "\r\tClasses\r\t-------\r" 
    } 
    for {set i 1} {$i <= $scrp_params(nbClasses)} {incr i} {
	Scrp::class $i
    }
    # Comparison operators loop
    set scrp_params(nbCompOps) [Scrp::getNextVal 2]
    if {$scrp_params(nbCompOps)} {
	puts -nonewline $scrp_params(outfileId) "\r\tComparison Operators\r\t--------------------\r" 
    } 
    for {set i 1} {$i <= $scrp_params(nbCompOps)} {incr i} {
	Scrp::compop $i
    } 
    # Enumerations loop
    set scrp_params(nbEnums) [Scrp::getNextVal 2]
    if {$scrp_params(nbEnums)} {
	puts -nonewline $scrp_params(outfileId) "\r\tEnumerations\r\t------------\r" 
    } 
    for {set i 1} {$i <= $scrp_params(nbEnums)} {incr i} {
	Scrp::enums $i
    } 
}


proc Scrp::event {eventnum} {
    global scrp_params
    Scrp::outputLine "\r\t"
    Scrp::outputLine " -- "
    Scrp::checkAlignByte
    Scrp::outputLine " (" 4
    Scrp::outputLine "/" 4
    Scrp::outputLine ")\r\t\tReply (" 4
    Scrp::outputLine ") -- "
    Scrp::checkAlignByte
    puts -nonewline $scrp_params(outfileId) "\r\t\tReply is "
    Scrp::parseFlags
    Scrp::outputLine "\t\tDirect parameter type: " 4
    Scrp::outputLine " -- "
    Scrp::checkAlignByte
    puts -nonewline $scrp_params(outfileId) "\r\t\tIt is "
    Scrp::parseFlags 1 1
    set scrp_params(nbAddParms) [Scrp::getNextVal 2]
    if {$scrp_params(nbAddParms)} {
	puts -nonewline $scrp_params(outfileId) "\t\tAdditional Parameters" 
    } 
    # Additional parameters loop	
    for {set i 1} {$i <= $scrp_params(nbAddParms)} {incr i} {
	Scrp::addparms $i
    }
}


proc Scrp::addparms {parmnum} {
    global scrp_params
    Scrp::outputLine "\r\t\t  "
    Scrp::checkAlignByte
    Scrp::outputLine " (keyword " 4
    Scrp::outputLine ", type " 4
    Scrp::outputLine ")\r\t\t\t-- "
    Scrp::checkAlignByte
    puts -nonewline $scrp_params(outfileId) "\r\t\t\t-- It is "
    Scrp::parseFlags
}


proc Scrp::class {classnum} {
    global scrp_params
    Scrp::outputLine "\r\t "
    Scrp::checkAlignByte
    Scrp::outputLine " (" 4
    Scrp::outputLine ") -- "
    Scrp::checkAlignByte
    set scrp_params(nbPropr) [Scrp::getNextVal 2]
    if {$scrp_params(nbPropr)} {
	puts -nonewline $scrp_params(outfileId) "\r\t\tProperties" 
    } 
    # Class properties loop	
    for {set i 1} {$i <= $scrp_params(nbPropr)} {incr i} {
	Scrp::classpropr $i
    }
    set scrp_params(nbElems) [Scrp::getNextVal 2]
    if {$scrp_params(nbElems)} {
	puts -nonewline $scrp_params(outfileId) "\t\tElements" 
    } 
    # Class elements loop	
    for {set i 1} {$i <= $scrp_params(nbElems)} {incr i} {
	Scrp::elements $i
    }
}


proc Scrp::classpropr {propnum} {
    global scrp_params
    Scrp::outputLine "\r\t\t\t"
    Scrp::checkAlignByte
    Scrp::outputLine " (" 4
    Scrp::outputLine "/" 4
    Scrp::outputLine ") -- "
    Scrp::checkAlignByte
    puts -nonewline $scrp_params(outfileId) "\r\t\t\t\t-- "
    Scrp::parseFlags 0 2
}


proc Scrp::elements {elemnum} {
    global scrp_params
    Scrp::outputLine "\r\t\t\tClass element ID : " 4
    set scrp_params(nbKeyforms) [Scrp::getNextVal 2]
    if {$scrp_params(nbKeyforms)} {
	puts -nonewline $scrp_params(outfileId) "\r\t\t\t\t$scrp_params(nbKeyforms) Key Forms: " 
    } 
    # Key forms loop	
    for {set i 1} {$i <= $scrp_params(nbKeyforms)} {incr i} {
	Scrp::outputLine " " 4
    }
}


proc Scrp::compop {compopnum} {
    global scrp_params
    Scrp::outputLine "\t - "
    Scrp::checkAlignByte
    Scrp::outputLine "" 4
    Scrp::outputLine " -- "
    Scrp::checkAlignByte
}


proc Scrp::enums {enumnum} {
    global scrp_params
    Scrp::outputLine "\r\tEnum " 4
    set scrp_params(nbEnumerators) [Scrp::getNextVal 2]
    if {$scrp_params(nbEnumerators)} {
	puts -nonewline $scrp_params(outfileId) "\r\t\tEnumerators" 
    } 
    # Key enumerators loop	
    for {set i 1} {$i <= $scrp_params(nbEnumerators)} {incr i} {
	Scrp::enumerator $i
    }
}


proc Scrp::enumerator {enumerator} {
    global scrp_params
    Scrp::outputLine "\r\t\t  - "
    Scrp::checkAlignByte
    Scrp::outputLine " (" 4
    Scrp::outputLine ") -- "
    Scrp::checkAlignByte
}


proc Scrp::parseFlags { {firstflag 1} {lastflag 0} } {
    global scrp_params
    set flags [Scrp::convToBinary [Scrp::getNextVal]]
    if {[lindex $flags 1]} {
	puts -nonewline $scrp_params(outfileId) "a list, "
    } else {
	puts -nonewline $scrp_params(outfileId) "a single item, "
    }
    if {$firstflag} {
	if {[lindex $flags 0]} {
	    puts -nonewline $scrp_params(outfileId) "optional, "
	} else {
	    puts -nonewline $scrp_params(outfileId) "required, "
	}
    } 
    if {[lindex $flags 2]} {
	puts -nonewline $scrp_params(outfileId) "enumerated "
    } else {
	puts -nonewline $scrp_params(outfileId) "not enumerated "
    }
    if {$lastflag=="2"} {
	if {[lindex $flags 3]} {
	    puts -nonewline $scrp_params(outfileId) "\[r/w\]"
	} else {
	    puts -nonewline $scrp_params(outfileId) "\[r/o\]"
	}
    }
    puts -nonewline $scrp_params(outfileId) "\r"
    Scrp::getNextVal
}


proc Scrp::getPutString { {nbchar 0} {singq 0} } {
    global scrp_params aetevar
    if {$nbchar} {
	set l ""
	for {set i 0} {$i < $nbchar} {incr i} {
	    set beg $scrp_params(aetepointer)
	    set end [expr $scrp_params(aetepointer) + 1]
	    set scrp_params(aetepointer) [expr $scrp_params(aetepointer) + 2]	    
	    append l [format %c "0x[string range $aetevar $beg $end]"]
	}
	if {[expr [info tclversion] >= 8.0]} {
	    set l [encoding convertfrom macRoman $l]
	}
    } else {
	set nbc [Scrp::getNextVal]
	if {$nbc} {
	    Scrp::getPutString $nbc	    
	} else {
	    puts -nonewline $scrp_params(outfileId) ""
	}
	return
    }
    if {$singq==1} {
	set l "\'$l\'"
    } 
    puts -nonewline $scrp_params(outfileId) $l
}


proc Scrp::outputLine {title {nbc 0}} {
    global scrp_params
    puts -nonewline $scrp_params(outfileId) $title
    if {$nbc==4} {
        set singq 1
    } else {
	set singq 0
	}
    Scrp::getPutString $nbc $singq
}


proc Scrp::outputVal {title {nb 1} } {
    global scrp_params
    puts -nonewline $scrp_params(outfileId) "$title"
    puts -nonewline $scrp_params(outfileId) [Scrp::getNextVal $nb]
}


proc Scrp::checkAlignByte {} {
    global scrp_params
    if {[expr {$scrp_params(aetepointer)/2} % 2]} {
	Scrp::getNextVal
    }   
}


proc Scrp::convToBinary {arg} {
    set result {}
    while {$arg} {
	set rest [expr $arg % 2]
	lappend result $rest
	set arg [expr {$arg - $rest}]
	set arg [expr $arg / 2]
    }
    set result [lreverse $result]
    set diff [expr {8 - [llength $result]}]
    if {[expr $diff > 0]} {
	for {set i 0} {$i < $diff} {incr i} {
	    set result [linsert $result 0 0]
	}
    } 
    return $result
}


proc Scrp::getNextVal { {nb 1} } {
    global scrp_params aetevar
    set l [expr $nb*2]
    set beg $scrp_params(aetepointer)
    set end [expr $scrp_params(aetepointer) + $l -1]
    set scrp_params(aetepointer) [expr $scrp_params(aetepointer) + $l]
    return [format %i "0x[string range $aetevar $beg $end]"]
}



