## -*-Tcl-*- (auto-install)
 # ==========================================================================
 # 
 #  FILE: "autoSaveOnSwitch.tcl"
 #                                   created: 06/10/2001 {10:19:51 pm}
 #                               last update: 04/04/2002 {04:49:45 PM}
 #                               
 #  Description:
 #  
 #  Activating this feature via "Config --> Preferences --> Backups" will
 #  automatically save all windows which exist as local files when
 #  switching from Alpha to another application.
 # 
 #  Author: Craig Barton Upright
 #  E-mail: <cupright@princeton.edu>
 #    mail: Princeton University,  Department of Sociology
 #          Princeton, New Jersey  08544
 #    www: <http://www.princeton.edu/~cupright>
 #
 # --------------------------------------------------------------------------
 #  
 # Copyright (c) 2001-2002  Craig Barton Upright
 # 
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
 # 
 #   Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
 # 
 #   Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
 # 
 #   Neither the name of Alpha/Alphatk nor the names of its contributors may
 #    be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 # ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 # ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 # DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 # SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 # CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 # LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 # DAMAGE.
 # 
 # ==========================================================================
 ##

alpha::declare flag autoSaveOnSwitch 0.1.1 "global-only" {
    # Initialization script.
    
    # To automatically save all open windows which exists as local files
    # (i.e. not new buffers) whenever switching from Alpha to another
    # program, turn this item on||To disable the automatic saving of all
    # open windows when switching from Alpha to another program, turn this
    # item off \
    newPref flag autoSaveOnSwitch 0
    set autoSaveOnSwitch 0
    lunion flagPrefs(Backups) autoSaveOnSwitch
    set flagPrefs(Backups) [lsort -ignore $flagPrefs(Backups)]
} {
    # Activation script.
    set autoSaveOnSwitch 1
    hook::register   suspendHook {autoSave::saveOnSwitch}
} {
    # Deactivation script.
    set autoSaveOnSwitch 0
    hook::deregister suspendHook {autoSave::saveOnSwitch}
} maintainer {
    "Craig Barton Upright" <cupright@princeton.edu> <http://www.princeton.edu/~cupright/>
} help {
    Activating this feature via "Config --> Preferences --> Backups" will
    automatically save all windows which exist as local files when
    switching from Alpha to another application.  
} uninstall {
    this-file
} requirements {
    if {[set alpha::platform] != "alpha"} {
	if {[catch {alpha::package require Alpha 8.2d2}]} {
	    error "Only Alphatk 8.2d2 or newer call the suspendHook"
	}
    } 
}

namespace eval autoSave {}

proc autoSave::saveOnSwitch {args} {

    if {![llength [set windows [winNames -f]]]} {return} 
    
    # While we'd like to use 'saveAll' here, if there is a new file (i.e
    # one that hasn't not been saved to disk yet) the save dialog is pretty
    # messed up -- best to only save files that currently exist.
    
    set currentWindow [win::CurrentTail]
    foreach window $windows {
	if {![win::IsFile $window]} {continue}
	getWinInfo -w $window arr
	if {$arr(dirty) && ![catch {bringToFront $window}]} {
	    menu::fileProc "file" "save"
	}
    }
    catch {bringToFront $currentWindow}
}

# ===========================================================================
#
#  ------------  #
# 
#  Version History  #
# 
#  modified by  rev    reason
#  -------- --- ------ -----------
#  06-10-01 cbu 0.1    Original.
#  03-20-02 cbu 0.2    Split off from "backups.tcl" into separate package.
#                      

# ===========================================================================
#
# .