## -*-Tcl-*-
 # ###################################################################
 #  AlphaTcl - core Tcl engine
 # 
 #  FILE: "tags.tcl"
 #                                    created: 04/12/2002 {10:48:03 AM} 
 #                                last update: 04/12/2002 {07:11:37 PM} 
 #  Author: Vince Darley
 #  E-mail: vince@santafe.edu
 #          317 Paseo de Peralta, Santa Fe
 #     www: http://www.santafe.edu/~vince/
 #  
 # ###################################################################
 ##

alpha::extension tags 0.1 {
    tags::load
} help {
    See the tags section of the Alpha Manual for more information
}

namespace eval tags {}

proc tags::load {} {}

# Only link these variables in Alpha 7.
if {[info tclversion] < 8.0} {
    # File to use for Tag searches.
    newPref linkio-file tagFile [file join $HOME cTAGS]
    # The parenthesised regexp block to use for the func name
    newPref linkvariable funcPar 1
} else {
    # File to use for Tag searches.
    newPref io-file tagFile [file join $HOME cTAGS]
    # The parenthesised regexp block to use for the func name
    newPref variable funcPar 1
}

# Change tag files without asking the user.
newPref flag autoSwitchTagFile 1

lunion flagPrefs(Tags)		autoSwitchTagFile
lunion varPrefs(Tags)		funcPar tagFile
hook::register fileset-current tags::fileChanged

#  Tags API  #

# tags::defaultFind - prompt user for a function name and attempt 
#  to use the file 'cTAGS' to locate the function's 
#  definition
proc tags::defaultFind {} {
    if {[llength [winNames -f]]} {
	set name [getSelect]
    } else {
	set name ""
    }
    set name [prompt "Find which name?" $name]
    
    set tagFile [tagFileName]
    if {![catch {open $tagFile r} fin]} {
	while {![eof $fin]} {
	    set line [gets $fin]
	    if {[lindex $line 0] == $name} {
		close $fin
		set fl [lindex $line 1]
		file::openQuietly [lindex $fl 0]
		goto [rowColToPos [lindex $fl 1] 0]
		status::msg "Found function"
		return
	    }
	}
	close $fin
	status::msg "Couldn't find function."
    } else {
	status::msg "Couldn't read tag file."
    }
}
# tags::defaultCreateFile - searches all files in current file set 
#  and saves the locations of any function declarations
#  in a file called 'cTAGS'.
proc tags::defaultCreateFile {} {
    global tagFile funcExpr
    
    set expr [prompt {Function Expression:} $funcExpr]
    
    set tagscan [scancontext create]
    scanmatch $tagscan $expr {
	set func($matchInfo(submatch0)) [list $f $matchInfo(linenum)]
    }
    
    foreach f [getCurrFileSet] {
	if {![catch {open $f r} fid]} {
	    status::msg "scanning [file tail $f]"
	    scanfile $tagscan $fid
	    close $fid
	}
    }
    scancontext delete $tagscan
    
    status::msg "Writing tag file $tagFile"
    if {![catch {open $tagFile w} tagOut]} {
	foreach a [lsort [array names func]] {
	    puts $tagOut [list $a $func($a)]
	}
	close $tagOut
	status::msg "Done."
    } else {
	status::msg "Couldn't write tag file."
    }
}

# In Alpha7, we use the hard-coded versions by default.
if {[info tclversion] < 8.0 && ([info commands findTag] != "")} {
    rename tags::defaultFind {}
    rename tags::defaultCreateFile {}
    rename findTag tags::defaultFind
    rename createTagFile tags::defaultCreateFile
}

# Soon to be obsolete
proc findTag {} { tags::find}
proc createTagFile {} { tags::createFile }
proc tagFileName {} { tags::name }

proc tags::name {} {
    global gfileSets currFileSet 
    return [file join [lindex $gfileSets($currFileSet) 0] \
      "[join ${currFileSet}]TAGS"]
}

proc tags::find {} {
    global gfileSetsType currFileSet
    # try a type-specific method first
    if {[catch {fileset::$gfileSetsType($currFileSet)::findTag}]} {
	tags::defaultFind
    }
}

proc tags::createFile {} {
    global gfileSetsType currFileSet tagFile
    set tagFile [tagFileName]
    prefs::modified tagFile
    
    # try a type-specific method first
    if {[catch {fileset::$gfileSetsType($currFileSet)::createTagFile}]} {
	tags::defaultCreateFile
    }
}

#  Helper functions  #

proc tags::fileChanged {args} {
    # Bring in the tags file for this fileset
    global tagFile autoSwitchTagFile
    set fname [tagFileName]
    if {[file exists $fname] && ($tagFile != $fname)} {
	if {$autoSwitchTagFile \
	  || [dialog::yesno "Use tag file from folder \"[file dirname $fname]\" ?"]} {
	    set tagFile $fname
	}
    }
}
