#!/bin/sh
#-*-tcl-*-
# the next line restarts using wish \
exec wish "$0" ${1+"$@"}

#####################################################################
#  Alphatk - the ultimate editor
# 
#  FILE: "alphatk.tcl"
#                                    created: 04/12/98 {22:45:38 PM} 
#                                last update: 05/20/2002 {11:44:11 AM}}} 
#  Author: Vince Darley
#  E-mail: vince.darley@kagi.com
#    mail: Flat 10, 98 Gloucester Terrace, London, W2 6HP
#     www: http://www.santafe.edu/~vince/Alphatk.html
#  
# Copyright (c) 1998-2001  Vince Darley
# 
# See the file "license.terms" for information on use and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
# 
# In particular, while this is 'open source', it is NOT free, and
# cannot be copied in full or in part except according to the terms
# of the license agreement.
# 
#####################################################################

# Just in case we're running with tclsh.  We want Tk!
package require Tk

# Set this to one if you want to have messages
# broadcast to stdout.
if {0} {
    proc echo {txt} {
	# Catch this in case stdout doesn't exist.
	catch {puts stdout $txt}
    }
} else {
    proc echo {txt} {}
}

set now [clock clicks]
wm withdraw .
update
# In case we started up in a strange way.
cd [file dirname [info script]]

if {([info tclversion] < 8.1) || ([info patchlevel] == 8.1)} {
    error "Sorry, we require Tcl version 8.1.1 or newer;\
      you have [info tclversion]"
    return
}

if {[catch {package require msgcat}]} {
    echo "The msgcat package is required if you want to use\
      a localised version of Alphatk"
    namespace eval msgcat {}
    proc msgcat::mc {str} { return $str }
    proc msgcat::mclocale {args} {}
    proc msgcat::mcunknown {locale string} { return $string }
} else {
    msgcat::mcload Localisation
    # auto_load
    catch {msgcat::mcunknown blah blah}
}

if {[catch {package require Tclx}]} {
    echo "Installing Tclx speeds up some operations inside Alphatk"
}
if {[catch {package require ftp}]} {
    echo "Install the ftp package (part of 'tcllib') to let Alphatk\
      update itself more simply;"
}
if {[info tclversion] < 8.3} {
    echo "Suggestion: upgrade to Tcl 8.3 or 8.4 and Alphatk will\
      run more quickly."
}

# For forwards compatibility with latest tcl/tk.
# The other possible value is 'aqua' which is true for MacOS X.
if {![info exists tcl_platform(windowingsystem)]} {
    switch -- $tcl_platform(platform) {
	"unix" { set tcl_platform(windowingsystem) "x11" }
	"windows" { set tcl_platform(windowingsystem) "win32" }
	"macintosh" { set tcl_platform(windowingsystem) "classic" }
    }
}

# For debugging, you can force this on here.
# console show

namespace eval alpha {}

if {[info commands quit] == ""} {
    rename exit quit
    proc exit {{returnCode ""}} {}

    # MacTk seems to have a bug if we exit using exit.
    # So instead we destroy the root window.  This might
    # be a better default for all platforms, to allow for
    # embedding of Alphatk, or perhaps starting up Alphatk
    # in an interpreter which forms part of a bigger application.
    proc quit {} {
	# We don't really have to do this, but it is nicer
	# of us to clean up the things we know about.
	catch {message "Exiting"}
	global alpha::unmount
	if {[info exists alpha::unmount]} {
	    vfs::zip::Unmount $alpha::unmount
	}
	destroy .
    }
}

namespace eval alpha {
    namespace eval index {}
    namespace eval cache {}
    #set haveBasicKeys 1
    proc checkFileExists {dir {isdir 0}} {
	if {![file exists $dir] || \
	  ($isdir && ![file isdirectory $dir]) \
	  || (!$isdir && ![file isfile $dir])} {
	    tk_messageBox -title "Fatal Alphatk error" -message \
	      "Alphatk cannot find its\
	      [expr {$isdir ? {directory} : {file}}]\
	      '[file join [pwd] $dir]'.\
	      You should reinstall Alphatk.  Goodbye."
	    quit
	}
    }
}

if {[file exists AlphaTcl.zip]} {
    if {![catch {
	package require vfs
	set alpha::unmount [vfs::zip::Mount AlphaTcl.zip AlphaTcl.zip]
	set HOME [file join [pwd] AlphaTcl.zip Alpha]
    }]} {
	# we mounted successfully
	echo "Mounted AlphaTcl vfs"
    }
}

if {![info exists HOME]} {
    alpha::checkFileExists [file join Alpha Tcl] 1

    # Do this stuff to deal with aliases/links more easily later on.
    # We want 'HOME' to point to the parent of 'Tcl'.
    set HOME [file join [pwd] Alpha Tcl]
    if {[file type $HOME] == "link"} {
	set HOME [file readlink $HOME]
    }
    set HOME [file dirname $HOME]
    # Fix windows path
    if {$tcl_platform(platform) == "windows"} {
	catch {set HOME [file attributes $HOME -longname]}
    }

    # If we are using a packaged-up AlphaTcl.zip, then mount that
    # as our home directory, if we have the vfs package available
    if {[file exists [file join $HOME AlphaTcl.zip]]} {
	if {![catch {
	    package require vfs
	    set alpha::unmount [vfs::zip::Mount \
	      [file join $HOME AlphaTcl.zip] [file join $HOME AlphaTcl.zip]]
	    set HOME [file join $HOME AlphaTcl.zip]
	}]} {
	    # we mounted successfully
	    echo "Mounted AlphaTcl vfs"
	}
    }
}

alpha::checkFileExists [file join $HOME AlphaCore] 1
alpha::checkFileExists [file join $HOME AlphaCore Images] 1

foreach imagefile [glob -directory [file join $HOME AlphaCore Images] *.gif] {
    image create photo [file root [file tail $imagefile]] -file $imagefile
}
unset imagefile

# The default encoding is ISOmacRoman if we're developing with cvs,
# but if we're just a user, we will have downloaded a macRoman encoding
# version.  
set alpha::defaultEncoding macRoman
set alpha::platform tk

# We need this in case anything goes wrong.
proc alertnote {message} {
    set root [focus]
    if {![string length $root]} {
	set root .
    }
    if {[winfo exists .startup]} {
	global tcl_platform
	if {$tcl_platform(platform) == "unix"} {
	    destroy .startup
	    set parent .
	} else {
	    set parent .startup
	}
    } else { 
	set parent $root
    }

    tk_messageBox -message $message -parent $parent
    update
}

alpha::checkFileExists [file join $HOME Tcl SystemCode Init initialize.tcl]

if {[catch [list source [file join $HOME Tcl SystemCode Init initialize.tcl]]]} {
    catch {puts stderr "Problems sourcing initialize.tcl"}
    global errorInfo
    catch {console show}
    catch {puts stderr $errorInfo}
}

if {$alpha::macos} {
    set alpha::modifier_keys [list "Command" "cmd" "Option" "opt"]
    set alpha::command_key "Command"
    set alpha::option_key "Option"
} else {
    switch -- $tcl_platform(platform) {
	"windows" {
	    set alpha::modifier_keys [list "Alt" "alt" "Meta" "meta"]
	    set alpha::command_key "Alt"
	    set alpha::option_key "Meta"
	    # Add these defaults in.
	    event add <<Paste>> <Alt-v>
	    event add <<Copy>> <Alt-c>
	    event add <<Cut>> <Alt-x>
	}
	"unix" -
	default {
	    set alpha::modifier_keys [list "Alt" "alt" "Meta" "meta"]
	    set alpha::command_key "Alt"
	    set alpha::option_key "Meta"
	}
    }
}

# Load in either byte-compiled or source scripts.
foreach _f {balloonHelp default alpha_menus alpha_commands alpha_macros \
  alpha_dialogs alpha_gui alpha_inter_app alpha_io \
  alpha_windows alpha_vars alpha_search \
  alpha_positions alpha_editing alpha_colouring alpha_dnd} {
    if {[file exists [file join $HOME AlphaCore ${_f}.tbc]]} {
	source [file join $HOME AlphaCore ${_f}.tbc]
    } else {
	source [file join $HOME AlphaCore ${_f}.tcl]
    }
}
foreach _f {enhancedTextwidget syntaxColouring multiSearch} {
    if {[file exists [file join $HOME AlphaCore TextWidget ${_f}.tbc]]} {
	source [file join $HOME AlphaCore TextWidget ${_f}.tbc]
    } else {
	source [file join $HOME AlphaCore TextWidget ${_f}.tcl]
    }
}
unset _f

proc alpha::setIcon {w} {}
set alpha::useMyIcons 0
switch -- $tcl_platform(platform) {
    "windows" {
	if {[catch {wm iconbitmap . -default [file join $HOME AlphaCore Alpha.icr]}]} {
	    if {![catch {package require Winico}]} {
		source [file join $HOME AlphaCore alpha_win_ico.tcl]
		set alpha::useMyIcons 1
		set alpha::icon [alpha::loadIcon \
		  [file join $HOME AlphaCore Alpha.icr]]
		proc alpha::setIcon {w} {
		    variable icon
		    variable bigIcon
		    variable smallIcon
		    winico setwindow $w $icon big   $bigIcon
		    winico setwindow $w $icon small $smallIcon
		}
		# This doesn't work here, because Tk over-rides it later!
		#alpha::setIcon .
	    } else {
		echo "Suggestion: install the 'Winico' package or\
		  Tk 8.3.3 or newer to let Alpha use nice icons\
		  for its windows."
	    }
	}
	
	source [file join $HOME AlphaCore alpha_win_print.tcl]
    }
    "macintosh" {
    }
    "unix" -
    default {
    }
}

proc alpha::foundPrefs {} {
    set already_running [alpha::initInterApplication]
    if {$already_running} {
	global PREFS HOME
	if {![file::pathStartsWith $PREFS $HOME]} {
	    if {[askyesno "Alphatk already appears to be running; are you sure you\
	      want to start a second copy?  Sharing certain kinds of preferences\
	      between two running copies could cause problems.\n\nSelect\
	      no to quit"] != "yes"} {
		quit
		return
	    }
	}
    }
}

alpha::checkFileExists [file join $HOME Tcl SystemCode] 1

namespace eval pos {}
proc pos::compare {args} {
    if {[lindex $args 0] == "-w"} {
	eval text_wcmd [list [lindex $args 1] compare] [lrange $args 2 end]
    } else {
	eval text_cmd compare $args
    }
}

proc pos::math {i args} { 
    if {[llength $args] < 2} {
	return [text_cmd index "$i [join $args ""]c"]
    } else {
	return [text_cmd index "$i +[eval expr $args]c"]
    }
}
proc pos::math {args} { 
    if {[lindex $args 0] == "-w"} {
	set w [lindex $args 1]
	set i [lindex $args 2]
	set args [lrange $args 3 end]
	if {[llength $args] < 2} {
	    return [text_wcmd $w index "$i [join $args ""]c"]
	} else {
	    return [text_wcmd $w index "$i +[eval expr $args]c"]
	}
    } else {
	set i [lindex $args 0]
	set args [lrange $args 1 end]
	if {[llength $args] < 2} {
	    return [text_cmd index "$i [join $args ""]c"]
	  } else {
	    return [text_cmd index "$i +[eval expr $args]c"]
	}
    }
}

proc pos::diff {p1 p2 args} {
    if {$p1 == "-w"} {
	set w  $p2
	set p1 [lindex $args 0]
	set p2 [lindex $args 1]
	if {[text_wcmd $w compare $p1 > $p2]} {
	    return [string length [text_wcmd $w get $p2 $p1]]
	} else {
	    return [string length [text_wcmd $w get $p1 $p2]]
	}
    } else {
	if {[text_cmd compare $p1 > $p2]} {
	    return [string length [text_cmd get $p2 $p1]]
	} else {
	    return [string length [text_cmd get $p1 $p2]]
	}
    }
}

namespace eval global {}

if {[catch alpha::PreGuiStartup]} {
    catch {puts stderr "Problems calling alpha::PreGuiStartup"}
    global errorInfo
    catch {puts stderr $errorInfo}
}

if {[info commands newPref] == ""} {
    error "No newPref command; very bad startup problem"
}

if {($tcl_platform(platform) == "unix" && $tcl_platform(os) == "Darwin")} {
    if {[info exists argv]} {
	set new_argv {}
	foreach str $argv {
	    if {![regexp {\-psn_0_[0-9]+} $str]} {
		lappend new_argv $str
	    }
	}
	set argv $new_argv
	catch {unset str}
	unset new_argv
    }
}

# Load in either byte-compiled or source scripts.
foreach _f {alpha_startup} {
    if {[file exists [file join $HOME AlphaCore ${_f}.tbc]]} {
	source [file join $HOME AlphaCore ${_f}.tbc]
    } else {
	source [file join $HOME AlphaCore ${_f}.tcl]
    }
}
unset _f

# Go up one level, so that default saves/opens are not inside Alphatk
cd [file dirname [pwd]]

echo "Startup took [expr {[clock clicks] - $now}] clicks"
unset now

